package at.tuwien.service;

import at.tuwien.BaseUnitTest;
import at.tuwien.config.DockerConfig;
import at.tuwien.config.IndexConfig;
import at.tuwien.config.ReadyConfig;
import at.tuwien.exception.AmqpException;
import at.tuwien.gateway.BrokerServiceGateway;
import at.tuwien.repository.jpa.DatabaseRepository;
import at.tuwien.service.impl.RabbitMqServiceImpl;
import at.tuwien.utils.AmqpUtils;
import com.rabbitmq.client.BuiltinExchangeType;
import com.rabbitmq.client.Channel;
import lombok.extern.log4j.Log4j2;
import org.junit.jupiter.api.AfterAll;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.extension.ExtendWith;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.boot.test.context.SpringBootTest;
import org.springframework.boot.test.mock.mockito.MockBean;
import org.springframework.test.context.junit.jupiter.SpringExtension;

import java.io.IOException;
import java.util.List;

import static org.junit.jupiter.api.Assertions.*;
import static org.mockito.Mockito.doThrow;
import static org.mockito.Mockito.when;

@Log4j2
@SpringBootTest
@ExtendWith(SpringExtension.class)
public class MessageQueueServiceUnitTest extends BaseUnitTest {

    @MockBean
    private ReadyConfig readyConfig;

    @MockBean
    private IndexConfig indexConfig;

    @MockBean
    private DatabaseRepository databaseRepository;

    @MockBean
    private BrokerServiceGateway brokerServiceGateway;

    @MockBean
    private Channel channel;

    @Autowired
    private RabbitMqServiceImpl messageQueueService;

    @Test
    public void createExchange_fails() throws IOException {

        /* mock */
        doThrow(IOException.class)
                .when(channel)
                .exchangeDeclare(DATABASE_1_EXCHANGE, BuiltinExchangeType.DIRECT, true);

        /* test */
        assertThrows(AmqpException.class, () -> {
            messageQueueService.createExchange(DATABASE_1, USER_1_PRINCIPAL);
        });
    }

    @Test
    public void deleteExchange_fails() throws IOException {

        /* mock */
        doThrow(IOException.class)
                .when(channel)
                .exchangeDelete(DATABASE_1_EXCHANGE);

        /* test */
        assertThrows(AmqpException.class, () -> {
            messageQueueService.deleteExchange(DATABASE_1);
        });
    }

}
