package at.tuwien.api.database.table.columns;

import at.tuwien.api.container.image.ImageDateDto;
import at.tuwien.api.database.table.columns.concepts.ConceptDto;
import at.tuwien.api.database.table.columns.concepts.UnitDto;
import com.fasterxml.jackson.annotation.JsonIgnore;
import com.fasterxml.jackson.annotation.JsonProperty;
import io.swagger.v3.oas.annotations.Parameter;
import io.swagger.v3.oas.annotations.media.Schema;
import lombok.*;
import lombok.extern.jackson.Jacksonized;
import org.springframework.data.elasticsearch.annotations.Document;

import jakarta.validation.constraints.NotBlank;
import jakarta.validation.constraints.NotNull;

@Getter
@Setter
@Builder
@NoArgsConstructor
@AllArgsConstructor
@Jacksonized
@ToString
@Document(indexName = "columnindex", createIndex = false)
public class ColumnDto {

    @JsonIgnore
    private Long containerId;

    @JsonIgnore
    private Long databaseId;

    @JsonIgnore
    private Long tableId;

    @NotNull
    private Long id;

    @NotBlank
    @Schema(example = "Date")
    private String name;

    @NotBlank
    @JsonProperty("internal_name")
    @Schema(example = "mdb_date")
    private String internalName;

    @JsonProperty("date_format")
    @org.springframework.data.annotation.Transient
    private ImageDateDto dateFormat;

    @NotNull
    @JsonProperty("auto_generated")
    @Schema(example = "false")
    private Boolean autoGenerated;

    @NotNull
    @JsonProperty("is_primary_key")
    @Schema(example = "true")
    private Boolean isPrimaryKey;

    @JsonProperty("index_length")
    private Integer indexLength;

    @NotNull
    @JsonProperty("column_type")
    @Schema(example = "string")
    private ColumnTypeDto columnType;

    @Schema(example = "255")
    private Integer length;

    private ConceptDto concept;

    private UnitDto unit;

    @NotNull
    @JsonProperty("is_public")
    @Schema(example = "true")
    private Boolean isPublic;

    @NotNull
    @JsonProperty("is_null_allowed")
    @Schema(example = "false")
    private Boolean isNullAllowed;

    @JsonProperty("enum_values")
    @Parameter(description = "enum values, only considered when type = ENUM")
    private String[] enumValues;

}
