package at.tuwien.entities.container.image;

import at.tuwien.entities.container.Container;
import jakarta.persistence.*;
import lombok.*;
import org.hibernate.annotations.GenericGenerator;
import org.springframework.data.annotation.CreatedDate;
import org.springframework.data.annotation.LastModifiedDate;
import org.springframework.data.jpa.domain.support.AuditingEntityListener;

import java.time.Instant;
import java.util.List;

@Data
@Entity
@Builder
@ToString
@AllArgsConstructor
@NoArgsConstructor
@EntityListeners(AuditingEntityListener.class)
@EqualsAndHashCode(onlyExplicitlyIncluded = true)
@Table(name = "mdb_images", uniqueConstraints = @UniqueConstraint(columnNames = {"repository", "tag"}))
public class ContainerImage {

    @Id
    @EqualsAndHashCode.Include
    @GeneratedValue(generator = "images-sequence")
    @GenericGenerator(name = "images-sequence", strategy = "increment")
    @Column(updatable = false, nullable = false)
    public Long id;

    @Column(nullable = false)
    private String repository;

    @Column(nullable = false, columnDefinition = "TEXT default 'docker.io/library'")
    private String registry;

    @Column(nullable = false)
    private String tag;

    @Column(nullable = false)
    private String driverClass;

    @Column(nullable = false)
    private String dialect;

    @Column(nullable = false)
    private String jdbcMethod;

    @Column
    private String hash;

    @Column(columnDefinition = "TIMESTAMP")
    private Instant compiled;

    @Column
    private Long size;

    @Column(nullable = false)
    private Integer defaultPort;

    @ToString.Exclude
    @OneToMany(fetch = FetchType.LAZY, cascade = CascadeType.ALL)
    @JoinColumn(name = "iid", insertable = false, updatable = false)
    private List<ContainerImageEnvironmentItem> environment;

    @OneToMany(fetch = FetchType.LAZY, cascade = CascadeType.ALL, mappedBy = "image")
    private List<ContainerImageDate> dateFormats;

    @ToString.Exclude
    @org.springframework.data.annotation.Transient
    @OneToMany(fetch = FetchType.LAZY, mappedBy = "image")
    private List<Container> containers;

    @CreatedDate
    @Column(nullable = false, updatable = false, columnDefinition = "TIMESTAMP")
    private Instant created;

    @LastModifiedDate
    @Column(columnDefinition = "TIMESTAMP")
    private Instant lastModified;

    @PreRemove
    public void preRemove() {
        this.containers.forEach(container -> {
            container.setImage(null);
        });
    }

}
