package at.tuwien.mapper;

import at.tuwien.api.database.ViewBriefDto;
import at.tuwien.api.database.ViewCreateDto;
import at.tuwien.api.database.ViewDto;
import at.tuwien.entities.database.View;
import at.tuwien.exception.QueryMalformedException;
import org.mapstruct.Mapper;
import org.mapstruct.Mapping;
import org.mapstruct.Mappings;
import org.mapstruct.Named;

import java.sql.*;
import java.text.Normalizer;
import java.util.Locale;
import java.util.regex.Pattern;

@Mapper(componentModel = "spring", uses = {ContainerMapper.class, UserMapper.class})
public interface ViewMapper {

    org.slf4j.Logger log = org.slf4j.LoggerFactory.getLogger(ViewMapper.class);

    @Named("internalMapping")
    default String nameToInternalName(String data) {
        if (data == null || data.length() == 0) {
            return data;
        }
        final Pattern NONLATIN = Pattern.compile("[^\\w-]");
        final Pattern WHITESPACE = Pattern.compile("[\\s]");
        String nowhitespace = WHITESPACE.matcher(data).replaceAll("_");
        String normalized = Normalizer.normalize(nowhitespace, Normalizer.Form.NFD);
        String slug = NONLATIN.matcher(normalized).replaceAll("");
        final String name = slug.toLowerCase(Locale.ENGLISH);
        log.trace("mapped name {} to internal name {}", data, name);
        return name;
    }

    @Mappings({
            @Mapping(target = "database.container", ignore = true)
    })
    ViewDto viewToViewDto(View data);

    ViewBriefDto viewToViewBriefDto(View data);

    default PreparedStatement viewToRawDeleteViewQuery(Connection connection, View view)
            throws QueryMalformedException {
        log.debug("mapping delete view query, view={}", view);
        final StringBuilder statement = new StringBuilder("DROP VIEW `v_")
                .append(nameToInternalName(view.getName()))
                .append("`;");
        try {
            log.trace("mapped delete view {} to prepared statement", view.getName());
            return connection.prepareStatement(statement.toString());
        } catch (SQLException e) {
            log.debug("Failed to prepare statement {}, reason: {}", statement, e.getMessage());
            throw new QueryMalformedException("Failed to prepare statement", e);
        }
    }

    default PreparedStatement viewCreateDtoToRawCreateViewQuery(Connection connection, ViewCreateDto data)
            throws QueryMalformedException {
        log.debug("mapping create view, data={}", data);
        final StringBuilder statement = new StringBuilder("CREATE VIEW `")
                .append(nameToInternalName(data.getName()))
                .append("` AS (")
                .append(data.getQuery())
                .append(")");
        try {
            final PreparedStatement pstmt = connection.prepareStatement(statement.toString());
            log.trace("mapped create view {} to prepared statement {}", data.getName(), pstmt);
            return pstmt;
        } catch (SQLException e) {
            log.error("Failed to prepare statement {}, reason: {}", statement, e.getMessage());
            throw new QueryMalformedException("Failed to prepare statement", e);
        }
    }

}
